from mindsdb_sdk.classes.proxy import Proxy
from mindsdb_sdk.classes.datasources import DataSources
from mindsdb_sdk.classes.predictors import Predictors
from mindsdb_sdk.classes.config import Config

class SDK():
    def __init__(self, host, user=None, password=None, token=None):
        self.proxy = Proxy(host, user, password, token)
        if self.proxy.ping():
            print(f'Connected to mindsdb host: {host} !')
        else:
            print(f'Failed to connect to mindsdb host: {host} !') 

        self.datasources = DataSources(self.proxy)
        self.predictors = Predictors(self.proxy)
        self.confg = Config(self.proxy)


import os
import pandas as pd

def auto_ml_config(mode='native', connection_info=None):
    if mode == 'native':
        os.environ['MINDSDB_PANDAS_AUTOML_MODE'] = 'native'
    elif mode == 'api':
        os.environ['MINDSDB_PANDAS_AUTOML_MODE'] = 'api'
        os.environ['MINDSDB_PANDAS_AUTOML_HOST'] = connection_info['host']

        if 'user' in connection_info:
            os.environ['MINDSDB_PANDAS_AUTOML_USER'] = connection_info['user']

        if 'password' in connection_info:
            os.environ['MINDSDB_PANDAS_AUTOML_USER'] = connection_info['password']

        if 'token' in connection_info:
            os.environ['MINDSDB_PANDAS_AUTOML_USER'] = connection_info['token']
    else:
        raise Exeption(f'Invalid mode: {mode} for the pandas auto_ml accessor!')



@pd.api.extensions.register_dataframe_accessor("auto_ml")
class AutoML:
    def __init__(self, pandas_obj):
        self._df = pandas_obj
        self._predictor = None
        self._analysis = None
        self.mode = os.environ['MINDSDB_PANDAS_AUTOML_MODE']
        if self.mode == 'api':
            self.host = os.environ['MINDSDB_PANDAS_AUTOML_HOST']
            self.user = os.environ.get('MINDSDB_PANDAS_AUTOML_USER', None)
            self.password = os.environ.get('MINDSDB_PANDAS_AUTOML_PASSWORD', None)
            self.token = os.environ.get('MINDSDB_PANDAS_AUTOML_TOKEN', None)

    @property
    def analysis(self):
        if self.mode == 'native':
            from mindsdb_native.libs.controllers.functional import analyse_dataset
            if self._analysis is None:
                self._analysis = analyse_dataset(self._df)
            return self._analysis
        else:
            raise Exception('API mode not supported for this call yet!')

    def learn(self, to_predict, name=None):
        if self.mode == 'native':
            from mindsdb_native import Predictor

            if name is None:
                name = str(pd.util.hash_pandas_object(self._df).sum())

            self._predictor = Predictor(name)
            self._predictor.learn(from_data=self._df, to_predict=to_predict)

            return name
        else:
            raise Exception('API mode not supported for this call yet!')


    def predict(self, name=None):
        if self.mode == 'native':
            from mindsdb_native import Predictor
            
            if name is not None:
                predictor = Predictor(name)
            else:
                predictor = self._predictor
            return predictor.predict(when_data=self._df)
        else:
            raise Exception('API mode not supported for this call yet!')



import requests
import os


class Proxy(object):

    def __init__(self, host, user=None, password=None, token=None) -> None:
        self._host = host.rstrip('/')
        self._apikey = None

        if token is not None:
            self._apikey = token
        if user is not None:
            self._apikey = self.post('/api/login', json={'email': user, 'password': password})['token']


    def post(self, route, data=None, json=None, params=None):
        if params is None:
            params = {}
        if self._apikey is not None:
            params['apiKey'] = self._apikey

        if data is not None:
            response = requests.post(self._host + '/api' + route, data=data, params=params)
        elif json is not None:
            print(1, route, json)
            response = requests.post(self._host + '/api' + route, json=json, params=params)
        else:
            response = requests.post(self._host + '/api' + route, params=params)

        if response.status_code != 200:
            raise Exception(f'Error({response.status_code}) with message: {response.text} !')

        return response.json()

    def put(self, route, data=None, json=None, params=None, files=None):
        if params is None:
            params = {}
        if self._apikey is not None:
            params['apiKey'] = self._apikey

        if files is not None:
            del_tmp = False
            if 'df' in files:
                del_tmp = True
                files['df'].to_csv('tmp_upload_file.csv', index=False)
                files['file'] = 'tmp_upload_file.csv'
                del files['df']

            with open(files['file'],'rb') as fp:
                files['file'] = fp
                data = {}
                data['source_type'] = 'file'

                response = requests.put(self._host + '/api' + route, files=files, data=data)

            if del_tmp:
                os.remove('tmp_upload_file.csv')


        elif data is not None:
            response = requests.put(self._host + '/api' + route, data=data, params=params)
        elif json is not None:
            response = requests.put(self._host + '/api' + route, json=json, params=params)
        else:
            response = requests.put(self._host + '/api' + route, params=params)

        if response.status_code != 200:
            raise Exception(f'Error({response.status_code}) with message: {response.text} !')

        return response.json()

    def get(self, route, params=None):
        if params is None:
            params = {}
        if self._apikey is not None:
            params['apiKey'] = self._apikey

        response = requests.get(self._host + '/api' + route, params=params)
        if response.status_code != 200:
            raise Exception(f'Error({response.status_code}) with message: {response.text} !')
        return response.json()

    def delete(self, route, params=None):
        if params is None:
            params = {}
        if self._apikey is not None:
            params['apiKey'] = self._apikey

        response = requests.delete(self._host + '/api' + route, params=params)

        if response.status_code != 200:
            raise Exception(f'Error({response.status_code}) with message: {response.text} !')

        return response.json()

    def ping(self):
        try:
            return self.get('/util/ping')['status'] == 'ok'
        except Exception as _:
            return False



import time


class DataSource():
    def __init__(self, proxy, name):
        self._proxy = proxy
        self.name = name
        self._analysis = None

    def get_info(self):
        return self._proxy.get(f'/datasources/{self.name}')

    @property
    def analysis(self, wait_seconds=360):
        if self._analysis is None:
            analysis = self._proxy.get(f'/datasources/{self.name}/analyze')
            for i in range(wait_seconds):
                if 'status' in analysis and analysis['status'] == 'analyzing':
                    time.sleep(10)
                analysis = self._proxy.get(f'/datasources/{self.name}/analyze')

            if 'status' in analysis and analysis['status'] == 'analyzing':
                raise Exception(f'Analysis not yet ready after waiting for {wait_seconds}, consider setting the `wait_seconds` to a higher value or reporting a bug if you think this should not take as long as it does for your dataset.')
            self._analysis = analysis
        return self._analysis

    def __iter__(self):
        return iter(list(self.analysis.keys()))

    def __len__(self):
        return len(list(self.analysis.keys()))

    def __getitem__(self, k):
        return self.analysis[k]

    def __delete__(self):
        self._proxy.delete(f'/datasources/{self.name}')


class DataSources():
    def __init__(self, proxy):
        self._proxy = proxy

    def list_info(self):
        return self._proxy.get('/datasources')

    def list_datasources(self):
        return [DataSource(self._proxy, x['name']) for x in self._proxy.get('/datasources')]

    def __getitem__(self, name):
        return DataSource(self._proxy, name)

    def __len__(self) -> int:
        return len(self.list_datasources())

    def __delitem__(self, name):
        self._proxy.delete(f'/datasources/{name}')

    def __setitem__(self, name, params):
        '''
        params is a dictionary that can contain:
        * file - File path
        * df - pandas dataframe
        * ulr - Url to file
        * source - file | url | <integration id>
        * source_type - file | ??

        and if source == <integration id>:
            * query
            + additional integration specific params (see docs in mindsdb)
        '''
        files = {}
        data = {}
        for k in params:
            if k in ['file', 'df']:
                files[k] = params[k]
            else:
                data[k] = params[k]
        if len(files) == 0:
            files = None
        self._proxy.put(f'/datasources/{name}', files=files, data=data)



import setuptools


about = {}
with open("mindsdb_sdk/__about__.py") as fp:
    exec(fp.read(), about)


with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as req_file:
    requirements = req_file.read().splitlines()

setuptools.setup(
    name=about['__title__'],
    version=about['__version__'],
    url=about['__github__'],
    download_url=about['__pypi__'],
    license=about['__license__'],
    author=about['__author__'],
    author_email=about['__email__'],
    description=about['__description__'],
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    install_requires=requirements,
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    python_requires=">=3.6"
)



import time


class Predictor():
    def __init__(self, proxy, name):
        self._proxy = proxy
        self.name = name
        ok = False
        for i in range(8):
            # @TODO: Remove once https://github.com/mindsdb/mindsdb/issues/916 is fixed
            time.sleep(1)
            try:
                self.get_info()
                ok = True
                break
            except Exception as e:
                pass

        if not ok:
            raise Exception(f'Predictor {name} does not exist')

    def get_info(self):
        return self._proxy.get(f'/predictors/{self.name}')

    def delete(self):
        self._proxy.delete(f'/predictors/{self.name}')

    def predict(self, datasource, args=None):
        if args is None:
            args = {}
        if isinstance(datasource, str) or (isinstance(datasource, dict) and 'created_at' in datasource and 'updated_at' in datasource and 'name' in datasource):
            return self._proxy.post(f'/predictors/{self.name}/predict_datasource', json={
                'data_source_name':datasource
                ,'kwargs': args
            })
        else:
            return self._proxy.post(f'/predictors/{self.name}/predict', json={
                'when':datasource
                ,'kwargs': args
            })


class Predictors():
    def __init__(self, proxy):
        self._proxy = proxy

    def list_info(self):
        return self._proxy.get('/predictors')

    def  list_predictor(self):
        return [Predictor(self._proxy, x['name']) for x in self._proxy.get('/predictors')]

    def __getitem__(self, name):
        return Predictor(self._proxy, name)

    def __len__(self) -> int:
        return len(self.list_predictor())

    def __delitem__(self, name):
        self._proxy.delete(f'/predictors/{name}')

    def learn(self, name, datasource, to_predict, args=None):
        if args is None:
            args = {}
        datasource = datasource['name'] if isinstance(datasource,dict) else datasource
        self._proxy.put(f'/predictors/{name}', json={
            'data_source_name': datasource
            ,'kwargs': args
            ,'to_predict': to_predict
        })

    '''
    @TODO:
    * Add custom predictor
    * Fit custom predictor
    * Upload predictor
    * Download predictor
    * Rename predictor
    '''
